﻿using DataSharing.DataExchange;
using DataSharing.Enterprise;
using DataSharing.FwDataExchange;
using DataSharing.Share.Dtos;
using DataSharing.Share.Dtos.DataExchange;
using DataSharing.Share.Dtos.HotlineSetting;
using DotNetCore.CAP;
using Fw.Utility.UnifyResponse;
using Hotline.Api.Sdk;
using Hotline.Share.Dtos.Order;
using MapsterMapper;
using MediatR;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;
using System.Security.Cryptography;
using System.Text;
using XF.Domain.Filters;
using XF.Domain.Repository;

namespace DataSharing.Host.Controllers
{
    /// <summary>
    /// 市州数据交换
    /// </summary>
    public class CityDataExchangeController : BaseController
    {
        private readonly IMapper _mapper;
        private readonly IMediator _mediator;
        private readonly ICapPublisher _capPublisher;
        private readonly IChannelConfigurationManager _channelConfigurationManager;
        private readonly IRepository<DsReceiveDataExchangeData> _dsReceiveDataExchangeDataRepository;
        private readonly FwClient _fwClient;

        /// <summary>
        /// 
        /// </summary>
        /// <param name="mapper"></param>
        /// <param name="mediator"></param>
        /// <param name="capPublisher"></param>
        /// <param name="channelConfigurationManager"></param>
        /// <param name="dsReceiveDataExchangeDataRepository"></param>
        /// <param name="hotlineClient"></param>
        public CityDataExchangeController(IMapper mapper, IMediator mediator,
            ICapPublisher capPublisher,
            IChannelConfigurationManager channelConfigurationManager,
            IRepository<DsReceiveDataExchangeData> dsReceiveDataExchangeDataRepository,
            FwClient fwClient)
        {
            _mapper = mapper;
            _mediator = mediator;
            _capPublisher = capPublisher;
            _channelConfigurationManager = channelConfigurationManager;
            _dsReceiveDataExchangeDataRepository = dsReceiveDataExchangeDataRepository;
            _fwClient = fwClient;
        }

        /// <summary>
        /// 宜宾泸州数据互换
        /// </summary>
        /// <param name="cityDataReceiveDto">宜宾泸州数据互换</param>
        /// <returns></returns>
        [HttpPost("city_hand_data")]
        [AllowAnonymous]
        public async Task<OpenResponse> CityHandData([FromBody] CityDataReceiveDto cityDataReceiveDto)
        {
            // 验证
            if (cityDataReceiveDto is null)
                return OpenResponse.Ok(DataExchangeDeReponse.Failed(description: "数据解析失败"));

            //验证数据
            string strResult = cityDataReceiveDto.Validate();
            if (!string.IsNullOrEmpty(strResult))
                return OpenResponse.Ok(DataExchangeDeReponse.Failed(description: strResult));

            var cityHandOver = _channelConfigurationManager.GetConfigurationCityHandOver();
            //验证MD5
            strResult = Verification(cityDataReceiveDto.Code, cityDataReceiveDto.Source + cityHandOver.SecretKey);
            if (!string.IsNullOrEmpty(strResult))
                return OpenResponse.Ok(DataExchangeDeReponse.Failed("-900", strResult));

            //写入原始数据表
            var data = new DsReceiveDataExchangeData()
            {
                ExternalId = cityDataReceiveDto.FSHCode,
                ServiceInterface = "city_hand_data",
                ReceiveData = System.Text.Json.JsonSerializer.Serialize(cityDataReceiveDto)
            };
            data.Id = await _dsReceiveDataExchangeDataRepository.AddAsync(data, HttpContext.RequestAborted);

            Hotline.Share.Dtos.Order.AddOrderDto orderDto = new()
            {
                Title = cityDataReceiveDto.Title,
                ExternalId = cityDataReceiveDto.FSHCode,
                AcceptType = cityDataReceiveDto.PurTypeName,
                HotspotSpliceName = cityDataReceiveDto.ConTypeName,
                FromName = cityDataReceiveDto.LinkName,
                Contact = cityDataReceiveDto.Mobile,
                Address = cityDataReceiveDto.Address,
                Content = cityDataReceiveDto.Content,
                Source = Hotline.Share.Enums.Order.ESource.CityDataExchangeLz,
                FromGender = cityDataReceiveDto.Gender switch
                {
                    "1" => Hotline.Share.Enums.Order.EGender.Male,
                    "2" => Hotline.Share.Enums.Order.EGender.Female,
                    _ => Hotline.Share.Enums.Order.EGender.Unknown,
                },
                SourceChannel= "市州互转",
                SourceChannelCode= "SZHZ"
            };
            //调用工单创建接口
            //   await _capPublisher.PublishAsync(DataSharing.Share.Mq.EventNames.SharingOrderCreation, orderDto, cancellationToken: HttpContext.RequestAborted);

            //调用工单创建接口
            // var result = await _hotlineClient.ReceiveOrderAsync(orderDto, HttpContext.RequestAborted);
            var result = await _fwClient.RequestNoTokenAsync<HotlineClientResponse>("api/v1/Order/add-anonymous", "Post", System.Text.Json.JsonSerializer.Serialize(orderDto));

            if (result != null && result.code==0)
            {
                return OpenResponse.Ok(DataExchangeDeReponse.Success(result.result.no, result.result.password, "您已成功提交数据！"));
            }
            else
                return OpenResponse.Ok(DataExchangeDeReponse.Failed(description: "接口调用失败！"));
        }

        #region 密钥验证

        /// <summary>
        /// 密钥验证
        /// </summary>
        /// <param name="Code">密钥</param>
        private static string Verification(string Code, string Source)
        {
            string strJson = "";
            string strJJYLCode = GetMD5(Source);
            if (strJJYLCode.ToLower().Trim() != Code.ToLower().Trim())
            {
                strJson = "密钥错误";
                return strJson;
            }
            return "";
        }

        #endregion

        #region MD5 32位加密

        /// <summary>
        /// MD5 32位加密
        /// </summary>
        /// <param name="myString"></param>
        /// <returns></returns>
        private static string GetMD5(string myString)
        {
            string cl = myString;
            string pwd = "";
            try
            {
                MD5 md5 = MD5.Create();//实例化一个md5对像
                                       // 加密后是一个字节类型的数组，这里要注意编码UTF8/Unicode等的选择　
                byte[] s = md5.ComputeHash(Encoding.UTF8.GetBytes(cl));
                // 通过使用循环，将字节类型的数组转换为字符串，此字符串是常规字符格式化所得
                for (int i = 0; i < s.Length; i++)
                {
                    // 将得到的字符串使用十六进制类型格式。格式后的字符是小写的字母，如果使用大写（X）则格式后的字符是大写字符 
                    pwd = pwd + s[i].ToString("X2");
                }
            }
            catch (Exception)
            {
            }
            return pwd;
        }

        #endregion
    }
}
