﻿using DataSharing.Repository.Extensions;
using DataSharing.Share.Dtos.HotlineSetting;
using FluentValidation;
using FluentValidation.AspNetCore;
using Hotline.Api.Sdk;
using Hotline.Application;
using Mapster;
using Serilog;
using XF.Domain.Dependency;
using XF.Domain.Filters;
using XF.EasyCaching;
using XF.Utility.MQ;

namespace DataSharing.Host;

internal static class StartupExtensions
{
    const string CorsOrigins = "CorsOrigins";

    internal static WebApplication ConfigureServices(this WebApplicationBuilder builder)
    {
        var services = builder.Services;
        var configuration = builder.Configuration;

        services.AddHttpContextAccessor();

#if DEBUG
        builder.WebHost.UseUrls("http://*:50105");
#endif

        services.AddHttpClient();

        services.Configure<ChannelConfiguration>(d => configuration.GetSection(nameof(ChannelConfiguration)).Bind(d));

        // Add services to the container.
        services
            .BatchInjectServices()
            .RegisterRepository()
            .AddApplication()
            ;

        ////Authentication
        //services.RegisterAuthentication(configuration);

        services.AddControllers(options =>
            {
                options.Filters.Add<UnifyResponseFilter>();
                options.Filters.Add<UserFriendlyExceptionFilter>();
            })
            ;

        // Learn more about configuring Swagger/OpenAPI at https://aka.ms/aspnetcore/swashbuckle
        services.AddEndpointsApiExplorer();

        //swagger
        services.RegisterSwagger();

        /* CORS */
        services.RegisterCors(configuration, CorsOrigins);

        //mapster
        services.RegisterMapper();

        //mediatr
        services.AddMediatR(d => { d.RegisterServicesFromAssembly(typeof(ApplicationStartupExtensions).Assembly); });

        //sqlsugar
        services.AddSqlSugar(configuration);

        //关闭TS时间
        AppContext.SetSwitch("Npgsql.EnableLegacyTimestampBehavior", true);
        AppContext.SetSwitch("Npgsql.DisableDateTimeInfinityConversions", true);

        //cache
        services.AddCache(d =>
        {
            var cacheConfig = configuration.GetSection("Cache").Get<CacheOptions>();
            cacheConfig.Adapt(d);
            d.Prefix = "DataSharing";
            d.TopicName = "Dsharing-topic";
        });

        //validator
        services.AddFluentValidationAutoValidation(config => { config.DisableDataAnnotationsValidation = true; })
            .AddValidatorsFromAssembly(typeof(ApplicationStartupExtensions).Assembly);

        //mq
        services.AddMq(configuration);

        //HotlineSdk
        var hotlineSdkConfig = configuration.GetRequiredSection("HotlineSdk").Get<HotlineSdkConfiguration>();
        services.AddHotlineSdk(hotlineSdkConfig.BaseUrl, hotlineSdkConfig.ApiKey, hotlineSdkConfig.ApiSecret);

        ////retry pipeline
        //services.AddResiliencePipeline(StrategyDefaults.RetryStrategy, d =>
        //{
        //    d.AddRetry(new RetryStrategyOptions
        //        {
        //            Delay = TimeSpan.FromSeconds(5),
        //        //BackoffType = DelayBackoffType.Constant,
        //        //// UseJitter = true,
        //        //MaxDelay = TimeSpan.FromSeconds(10),
        //        MaxRetryAttempts = 10
        //    });
        //        //.AddTimeout(TimeSpan.FromSeconds(5));
        //});

        //job
        services.RegisterJob();

        //services.AddSingleton<IAuthorizationPolicyProvider, AuthorizationPolicyProvider>();
        //services.AddSingleton<IAuthorizationHandler, PermissionHandler>();

        return builder.Build();
    }

    internal static WebApplication ConfigurePipelines(this WebApplication app)
    {
        app.UseSerilogRequestLogging();

        // Configure the HTTP request pipeline.
        var swaggerEnable = app.Configuration.GetSection("Swagger").Get<bool>();
        if (swaggerEnable)
        {
            app.UseSwagger();
            app.UseSwaggerUI();
        }

        app.UseCors(CorsOrigins);

        app.UseAuthentication();
        app.UseAuthorization();

        app.MapControllers()
            .RequireAuthorization();
        //app.MapSubscribeHandler();

        return app;
    }
}